--
-- Nugent Bale Fork
-- LS_UK Modteam
-- 07/06/2011
--

Nugent = {};

function Nugent.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Attachable, specializations);
end;

function Nugent:load(xmlFile)	

	self.setFork = SpecializationUtil.callSpecializationsFunction("setFork");
	self.setBaleTop = SpecializationUtil.callSpecializationsFunction("setBaleTop");
	self.setBaleCenter = SpecializationUtil.callSpecializationsFunction("setBaleCenter");
	self.setBaleLeft = SpecializationUtil.callSpecializationsFunction("setBaleLeft");
	self.setBaleRight = SpecializationUtil.callSpecializationsFunction("setBaleRight");
	self.objectInRange = Nugent.objectInRange;

	self.fork1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.tubes#index1"));
	self.fork2 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.tubes#index2"));
	self.fork3 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.tubes#index3"));
	self.forkId = 0;

	self.lockTop = {};
	self.lockTop.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.locknodeTop#index"));
	self.lockTop.child = getChildAt(self.lockTop.node, 0);
	self.lockTop.baleId = nil;
	
	self.lockCenter = {};
	self.lockCenter.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.locknodeCenter#index"));
	self.lockCenter.child = getChildAt(self.lockCenter.node, 0);
	self.lockCenter.baleId = nil;

	self.lockLeft = {};
	self.lockLeft.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.locknodeLeft#index"));
	self.lockLeft.child = getChildAt(self.lockLeft.node, 0);
	self.lockLeft.baleId = nil;
	
	self.lockRight = {};
	self.lockRight.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.locknodeRight#index"));
	self.lockRight.child = getChildAt(self.lockRight.node, 0);
	self.lockRight.baleId = nil;

	self.setArmOne = SpecializationUtil.callSpecializationsFunction("setArmOne");
	self.ArmOneAnimation = getXMLString(xmlFile, "vehicle.armOne#animationName");
	self.ArmOne = false;

	self.setArmTwo = SpecializationUtil.callSpecializationsFunction("setArmTwo");
	self.ArmTwoAnimation = getXMLString(xmlFile, "vehicle.armTwo#animationName");
	self.ArmTwo = false;	
end;

function Nugent:delete()
	for i, baleServerId in ipairs(self.lockTop) do
		local bale = networkGetObject(baleServerId);
		if bale ~= nil then
			if bale.nodeId ~= 0 then
				unmount(bale.nodeId);
			end;
		end;
	end;
end;

function Nugent:onDetach()
	self:setBaleTop(nil, false);
	self:setBaleCenter(nil, false);
	self:setBaleLeft(nil, false);
	self:setBaleRight(nil, false);
end;

function Nugent:readStream(streamId, connection)
	self:setFork(streamReadInt32(streamId), true);
	self:setBaleTop(streamReadInt32(streamId), true);
	self:setBaleCenter(streamReadInt32(streamId), true);
	self:setBaleLeft(streamReadInt32(streamId), true);
	self:setBaleRight(streamReadInt32(streamId), true);
	self:setArmOne(streamReadBool(streamId), true);
	self:setArmTwo(streamReadBool(streamId), true);	
end;

function Nugent:writeStream(streamId, connection)
	streamWriteInt32(streamId, self.forkId);
	streamWriteInt32(streamId, self.lockTop.baleId);
	streamWriteInt32(streamId, self.lockCenter.baleId);
	streamWriteInt32(streamId, self.lockLeft.baleId);
	streamWriteInt32(streamId, self.lockRight.baleId);
	streamWriteBool(streamId, self.ArmOne);
	streamWriteBool(streamId, self.ArmTwo);
end;

function Nugent:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Nugent:keyEvent(unicode, sym, modifier, isDown)
end;

function Nugent:update(dt)
	
	if self:getIsActive() then
		if self:getIsActiveForInput() then	
			if self.lockTop.baleId == nil and self.lockLeft.baleId == nil and self.lockRight.baleId == nil and self.lockCenter.baleId == nil then
				if InputBinding.hasEvent(InputBinding.TUBES) then
					self:setFork(math.abs(self.forkId - 1), false);
				end;
			end;
			
			if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) and self.lockTop.baleId == nil and self.lockLeft.baleId == nil and self.lockRight.baleId == nil and self.lockCenter.baleId == nil then
				self:setArmOne(not self.isArmOneOn);
			end;

			if self.forkId == 0 and self.lockTop.node ~= nil then 
				if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA3) then
					if self.lockTop.baleId == nil and self.ArmOne == false then
						local bale = self:objectInRange(self.lockTop);
					
						if bale ~= nil then
							self:setBaleTop(networkGetObjectId(bale), false);
							self:setArmOne(true,false);
						end;
					elseif self.lockTop.baleId ~= nil and self.ArmOne == true then
						self:setBaleTop(nil, false);
						self:setArmOne(not self.isArmOneOn);
					end;
				end;	
			end;
			if self.forkId == 1 and self.lockCenter.node ~= nil then 
				if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA3) then
					if self.lockCenter.baleId == nil and self.ArmOne == true then
						local bale = self:objectInRange(self.lockCenter);
					
						if bale ~= nil then
							self:setBaleCenter(networkGetObjectId(bale), false);
						end;
					else
						self:setBaleCenter(nil, false);
					end;
				end;	
			end;
			if self.forkId == 1 and (self.lockLeft.node ~= nil or self.lockRight.node ~= nil or self.lockCenter.node ~= nil) and self.ArmOne == false then
				if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA3) then
					if self:objectInRange(self.lockLeft)~= nil or self:objectInRange(self.lockRight) ~= nil then
						if self:objectInRange(self.lockLeft) ~= nil then
							local bale = self:objectInRange(self.lockLeft);

							if bale ~= nil then
								self:setBaleLeft(networkGetObjectId(bale), false);
							end;
						end;	
						if self:objectInRange(self.lockRight) ~= nil then
							local bale = self:objectInRange(self.lockRight);
		
							if bale ~= nil then
							 	self:setBaleRight(networkGetObjectId(bale), false);
							end;
						end;
					else	
						if self.lockLeft.baleId ~= nil and self.lockRight.baleId ~= nil then
							self:setBaleLeft(nil, false);
							self:setBaleRight(nil, false);
						end;
						if self.lockLeft.baleId ~= nil and self:objectInRange(self.lockRight) == nil then
							self:setBaleLeft(nil, false);
						end;
						if self.lockRight.baleId ~= nil and self:objectInRange(self.lockLeft) == nil then
							self:setBaleRight(nil, false);
						end;
					end;
				end;
			end;
		end;
	end;
end;

function Nugent:updateTick(dt)
	if self:getIsActive() and self.isServer then
		if self.forkId == 1 and self.ArmOne == true then
			self:setArmTwo(true,false);
		else
			self:setArmTwo(false,true);
		end;
	end;
	if self.forkId == 0 then
		setVisibility(self.fork2, true);
	else
		setVisibility(self.fork2, false);
	end;
end;

function Nugent:draw()

	if self.lockTop.baleId == nil and self.lockLeft.baleId == nil and self.lockRight.baleId == nil and self.lockCenter.baleId == nil then
		g_currentMission:addHelpButtonText(g_i18n:getText("Tubes"), InputBinding.TUBES);
	end;

	if self.lockTop.baleId == nil and self.lockLeft.baleId == nil and self.lockRight.baleId == nil and self.lockCenter.baleId == nil and self.ArmOne == false then
		g_currentMission:addHelpButtonText(g_i18n:getText("Close"), InputBinding.IMPLEMENT_EXTRA2);
	else
		if self.lockTop.baleId == nil and self.lockLeft.baleId == nil and self.lockRight.baleId == nil and self.lockCenter.baleId == nil and self.ArmOne == true then
			g_currentMission:addHelpButtonText(g_i18n:getText("Open"), InputBinding.IMPLEMENT_EXTRA2);
		end;
	end;

	if self.forkId == 0 then
		if self.lockTop.baleId ~= nil then
			g_currentMission:addHelpButtonText(g_i18n:getText("Unlock"), InputBinding.IMPLEMENT_EXTRA3);
		else
			if self:objectInRange(self.lockTop) ~= nil and self.ArmOne == false then
				g_currentMission:addHelpButtonText(g_i18n:getText("Lock"), InputBinding.IMPLEMENT_EXTRA3);
			end;
		end;
	end;
	
	if self.forkId == 1 then
		if self.lockCenter.baleId ~= nil then
			g_currentMission:addHelpButtonText(g_i18n:getText("Unlock"), InputBinding.IMPLEMENT_EXTRA3);
		else
			if self:objectInRange(self.lockCenter) ~= nil and self.ArmOne == true then
				g_currentMission:addHelpButtonText(g_i18n:getText("Lock"), InputBinding.IMPLEMENT_EXTRA3);
			end;
		end;
	end;

	if self.forkId == 1 and self.lockLeft.node ~= nil and self.lockRight.node ~= nil and self.ArmOne == false then
		if self.lockLeft.baleId ~= nil and self:objectInRange(self.lockRight) == nil then
			g_currentMission:addHelpButtonText(g_i18n:getText("Unlock"), InputBinding.IMPLEMENT_EXTRA3);
		else
			if self.lockRight.baleId ~= nil and self:objectInRange(self.lockLeft) == nil then
				g_currentMission:addHelpButtonText(g_i18n:getText("Unlock"), InputBinding.IMPLEMENT_EXTRA3);
			else
				if self.lockLeft.baleId ~= nil and self.lockRight.baleId ~= nil then
					g_currentMission:addHelpButtonText(g_i18n:getText("Unlock_Both"), InputBinding.IMPLEMENT_EXTRA3);
				else
					if self:objectInRange(self.lockLeft) ~= nil or self:objectInRange(self.lockRight) ~= nil then
						g_currentMission:addHelpButtonText(g_i18n:getText("Lock"), InputBinding.IMPLEMENT_EXTRA3);
					end;
				end;
			end;
		end;
	end;		
end;

function Nugent:setFork(fork, noEventSend)
	SetForksEvent.sendEvent(self, fork, noEventSend);

	setVisibility(self.fork1, fork == 0);
	setVisibility(self.fork3, fork == 1);
	self.forkId = fork;
end;

function Nugent:setBaleTop(baleId, noEventSend)
	if baleId ~= nil then
		if self.lockTop.child ~= nil then
			local bale = nil;

			if self.lockTop.baleId ~= nil then
				bale = networkGetObject(self.lockTop.baleId);
				if bale ~= nil then
					bale:unmount();
					self.lockTop.baleId = nil;
				end;
			end;

			BaleTopEvent.sendEvent(self, baleId, noEventSend);

			bale = networkGetObject(baleId);
			if bale ~= nil then
				if getUserAttribute(bale.nodeId, "isRoundbale") then
					bale:mount(self, self.lockTop.child, 0,0,0, 0,0,0);
				end;

				self.lockTop.baleId = baleId;
			end;
		end;
	else
		BaleTopEvent.sendEvent(self, baleId, noEventSend);

		if self.lockTop.baleId ~= nil then
			local bale = networkGetObject(self.lockTop.baleId);

			if bale ~= nil then
				bale:unmount();
				self.lockTop.baleId = nil;
			end;
		end;
	end;
end;

function Nugent:setBaleCenter(baleId, noEventSend)
	if baleId ~= nil then
		if self.lockCenter.child ~= nil then
			local bale = nil;

			if self.lockCenter.baleId ~= nil then
				bale = networkGetObject(self.lockCenter.baleId);
				if bale ~= nil then
					bale:unmount();
					self.lockCenter.baleId = nil;
				end;
			end;

			BaleCenterEvent.sendEvent(self, baleId, noEventSend);

			bale = networkGetObject(baleId);
			if bale ~= nil then
				if getUserAttribute(bale.nodeId, "isRoundbale") then
					bale:mount(self, self.lockCenter.child, 0,0,0, 0,0,0);
				else
					bale:mount(self, self.lockCenter.child, 0,0,-0.077, 0,math.pi/2,0);
				end;

				self.lockCenter.baleId = baleId;
			end;
		end;
	else
		BaleCenterEvent.sendEvent(self, baleId, noEventSend);

		if self.lockCenter.baleId ~= nil then
			local bale = networkGetObject(self.lockCenter.baleId);

			if bale ~= nil then
				bale:unmount();
				self.lockCenter.baleId = nil;
			end;
		end;
	end;
end;

function Nugent:setBaleLeft(baleId, noEventSend)
	if baleId ~= nil then
		if self.lockLeft.child ~= nil then
			local bale = nil;

			if self.lockLeft.baleId ~= nil then
				bale = networkGetObject(self.lockLeft.baleId);
				if bale ~= nil then
					bale:unmount();
					self.lockLeft.baleId = nil;
				end;
			end;

			BaleLeftEvent.sendEvent(self, baleId, noEventSend);

			bale = networkGetObject(baleId);
			if bale ~= nil then
				if getUserAttribute(bale.nodeId, "isRoundbale") then
					bale:mount(self, self.lockLeft.child, 0,0,0, 0,0,0);
				end;

				self.lockLeft.baleId = baleId;
			end;
		end;
	else
		BaleLeftEvent.sendEvent(self, baleId, noEventSend);

		if self.lockLeft.baleId ~= nil then
			local bale = networkGetObject(self.lockLeft.baleId);

			if bale ~= nil then
				bale:unmount();
				self.lockLeft.baleId = nil;
			end;
		end;
	end;
end;

function Nugent:setBaleRight(baleId, noEventSend)
	if baleId ~= nil then
		if self.lockRight.child ~= nil then
			local bale = nil;

			if self.lockRight.baleId ~= nil then
				bale = networkGetObject(self.lockRight.baleId);
				if bale ~= nil then
					bale:unmount();
					self.lockRight.baleId = nil;
				end;
			end;

			BaleRightEvent.sendEvent(self, baleId, noEventSend);

			bale = networkGetObject(baleId);
			if bale ~= nil then
				if getUserAttribute(bale.nodeId, "isRoundbale") then
					bale:mount(self, self.lockRight.child, 0,0,0, 0,0,0);
				end;

				self.lockRight.baleId = baleId;
			end;
		end;
	else
		BaleRightEvent.sendEvent(self, baleId, noEventSend);

		if self.lockRight.baleId ~= nil then
			local bale = networkGetObject(self.lockRight.baleId);

			if bale ~= nil then
				bale:unmount();
				self.lockRight.baleId = nil;
			end;
		end;
	end;
end;

function Nugent:objectInRange(refNode)
    local nearestDistance = 0.4;
    local nearestBale = nil;
    local px, py, pz = getWorldTranslation(refNode.node);

    for index, item in pairs(g_currentMission.itemsToSave) do
        if item.item:isa(Bale) or getUserAttribute(item.item.nodeId, "isStrawbale") or getUserAttribute(item.item.nodeId, "isHaybale") then
			local vx, vy, vz = getWorldTranslation(item.item.nodeId);
			local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);
			if distance < nearestDistance then
				nearestDistance = distance;
				nearestBale = item.item;
			end;
        end;
    end;

    return nearestBale;
end;

function Nugent:setArmOne(isArmOne,noEventSend)
	SetArmOneEvent.sendEvent(self, isArmOne, noEventSend);
	-- Play ArmOne animation --
	self.isArmOneOn = isArmOne;
	if self.isArmOneOn then
		if self.ArmOneAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.ArmOneAnimation, -1, nil, true);
			self.ArmOne = true;
		end;
	else
		if self.ArmOneAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.ArmOneAnimation, 1, nil, true);
			self.ArmOne = false;
		end;
	end;	
end;

function Nugent:setArmTwo(isArmTwo,noEventSend)
	SetArmTwoEvent.sendEvent(self, isArmTwo, noEventSend);
	-- Play ArmTwo animation --
	self.isArmTwoOn = isArmTwo;
	if self.isArmTwoOn then
		if self.ArmTwoAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.ArmTwoAnimation, -1, nil, true);
			self.ArmTwo = true;
		end;
	else
		if self.ArmTwoAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.ArmTwoAnimation, 1, nil, true);
			self.ArmTwo = false;
		end;
	end;	
end;


SetForksEvent = {};
SetForksEvent_mt = Class(SetForksEvent, Event);

InitEventClass(SetForksEvent, "SetForksEvent");

function SetForksEvent:emptyNew()
    local self = Event:new(SetForksEvent_mt);
    self.className="SetForksEvent";
    return self;
end;

function SetForksEvent:new(vehicle, fork)
    local self = SetForksEvent:emptyNew()
    self.vehicle = vehicle;
	self.fork = fork;
    return self;
end;

function SetForksEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
    self.fork = streamReadInt32(streamId);
    self.vehicle = networkGetObject(id);
    self:run(connection);
end;

function SetForksEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));	
	streamWriteInt32(streamId, self.fork);	
end;

function SetForksEvent:run(connection)
	self.vehicle:setFork(self.fork, true);
	if not connection:getIsServer() then
		g_server:broadcastEvent(SetForksEvent:new(self.vehicle, self.fork), nil, connection, self.object);
	end;
end;

function SetForksEvent.sendEvent(vehicle, fork, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetForksEvent:new(vehicle, fork), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetForksEvent:new(vehicle, fork));
		end;
	end;
end;

SetArmOneEvent = {};
SetArmOneEvent_mt = Class(SetArmOneEvent, Event);

InitEventClass(SetArmOneEvent, "SetArmOneEvent");

function SetArmOneEvent:emptyNew()
    local self = Event:new(SetArmOneEvent_mt);
    self.className="SetArmOneEvent";
    return self;
end;

function SetArmOneEvent:new(vehicle, isArmOne)
    local self = SetArmOneEvent:emptyNew()
    self.vehicle = vehicle;
	self.isArmOne = isArmOne;
    return self;
end;

function SetArmOneEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.isArmOne = streamReadBool(streamId);
    self.vehicle = networkGetObject(id);
    self:run(connection);
end;

function SetArmOneEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteBool(streamId, self.isArmOne);
end;

function SetArmOneEvent:run(connection)   
	self.vehicle:setArmOne(self.isArmOne, true);
    if not connection:getIsServer() then
        g_server:broadcastEvent(SetArmOneEvent:new(self.vehicle, self.isArmOne), nil, connection, self.vehicle);
    end;
end;

function SetArmOneEvent.sendEvent(vehicle, isArmOne, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetArmOneEvent:new(vehicle, isArmOne), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetArmOneEvent:new(vehicle, isArmOne));
		end;
	end;
end;

SetArmTwoEvent = {};
SetArmTwoEvent_mt = Class(SetArmTwoEvent, Event);

InitEventClass(SetArmTwoEvent, "SetArmTwoEvent");

function SetArmTwoEvent:emptyNew()
    local self = Event:new(SetArmTwoEvent_mt);
    self.className="SetArmTwoEvent";
    return self;
end;

function SetArmTwoEvent:new(vehicle, isArmTwo)
    local self = SetArmTwoEvent:emptyNew()
    self.vehicle = vehicle;
	self.isArmTwo = isArmTwo;
    return self;
end;

function SetArmTwoEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.isArmTwo = streamReadBool(streamId);
    self.vehicle = networkGetObject(id);
    self:run(connection);
end;

function SetArmTwoEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteBool(streamId, self.isArmTwo);
end;

function SetArmTwoEvent:run(connection)   
	self.vehicle:setArmTwo(self.isArmTwo, true);
    if not connection:getIsServer() then
        g_server:broadcastEvent(SetArmTwoEvent:new(self.vehicle, self.isArmTwo), nil, connection, self.vehicle);
    end;
end;

function SetArmTwoEvent.sendEvent(vehicle, isArmTwo, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetArmTwoEvent:new(vehicle, isArmTwo), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetArmTwoEvent:new(vehicle, isArmTwo));
		end;
	end;
end;

